@@ -45,6 +45,9 @@ def exec_process(cmdline, silent=True, input=None, **kwargs):
         sub = subprocess.Popen(args=cmdline, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
         stdout, stderr = sub.communicate(input=input)
         returncode = sub.returncode
+        if PY3:
+            stderr = stderr.decode('utf-8')
+            stdout = stdout.decode('utf-8')
         if not silent:
             sys.stdout.write(stdout)
             sys.stderr.write(stderr)
@@ -54,13 +57,7 @@ def exec_process(cmdline, silent=True, input=None, **kwargs):
         else:
             raise
     if returncode != 0:
-        if PY3:
-            stderr = stderr.decode('utf-8')
-            stdout = stdout.decode('utf-8')
-        output = (
-            'stderr:\n%s\nstdout:\n%s' % (stderr.rstrip("\n"),
-                                          stdout.rstrip("\n"))
-        )
+        output = 'stderr:\n%s\nstdout:\n%s' % (stderr.rstrip("\n"), stdout.rstrip("\n"))
         raise DistutilsError('Got return value %d while executing "%s", output was:\n%s' % (returncode, " ".join(cmdline), output))
     return stdout
 
@@ -114,7 +111,7 @@ class libuv_build_ext(build_ext):
     libuv_dir      = os.path.join('deps', 'libuv')
     libuv_repo     = 'https://github.com/libuv/libuv.git'
     libuv_branch   = 'v1.x'
-    libuv_revision = '413dbd8'
+    libuv_revision = '4ca78e9'
     libuv_patches  = []
 
     user_options = build_ext.user_options
@@ -137,6 +134,8 @@ class libuv_build_ext(build_ext):
     def build_extensions(self):
         self.force = self.force or self.libuv_force_fetch or self.libuv_clean_compile
         if self.use_system_libuv:
+            if sys.platform == 'win32':
+                raise DistutilsError('using a system provided libuv is unsupported on Windows')
             self.compiler.add_library('uv')
         else:
             if sys.platform == 'win32':
