@@ -29,7 +29,7 @@
 #ifdef PYUV_PYTHON3
 static PyModuleDef pyuv_module = {
     PyModuleDef_HEAD_INIT,
-    "pyuv",                 /*m_name*/
+    "pyuv._cpyuv",          /*m_name*/
     NULL,                   /*m_doc*/
     -1,                     /*m_size*/
     NULL,                   /*m_methods*/
@@ -81,7 +81,7 @@ init_pyuv(void)
 #ifdef PYUV_PYTHON3
     pyuv = PyModule_Create(&pyuv_module);
 #else
-    pyuv = Py_InitModule("pyuv", NULL);
+    pyuv = Py_InitModule("pyuv._cpyuv", NULL);
 #endif
 
     /* Errno module */
@@ -240,11 +240,7 @@ init_pyuv(void)
     UV_HANDLE_TYPE_MAP(XX)
 #undef XX
 
-    /* Module version (the MODULE_VERSION macro is defined by setup.py) */
-    PyModule_AddStringConstant(pyuv, "__version__", PYUV_STRINGIFY(MODULE_VERSION));
-
     /* libuv version */
-    PyModule_AddStringConstant(pyuv, "LIBUV_REVISION", PYUV_STRINGIFY(LIBUV_REVISION));
     PyModule_AddStringConstant(pyuv, "LIBUV_VERSION", uv_version_string());
 
     return pyuv;
@@ -259,13 +255,13 @@ fail:
 
 #ifdef PYUV_PYTHON3
 PyMODINIT_FUNC
-PyInit_pyuv(void)
+PyInit__cpyuv(void)
 {
     return init_pyuv();
 }
 #else
 PyMODINIT_FUNC
-initpyuv(void)
+init_cpyuv(void)
 {
     init_pyuv();
 }
