@@ -1,9 +1,11 @@
 
-from common import unittest2
+import unittest
+
+from common import TestCase
 import pyuv
 
 
-class MultiHandleTest(unittest2.TestCase):
+class MultiHandleTest(TestCase):
 
     def test_multihandle1(self):
         self.close_cb_called = 0
@@ -29,16 +31,15 @@ class MultiHandleTest(unittest2.TestCase):
             self.timer_cb_called += 1
             timer.stop()
             timer.close(close_cb)
-        loop = pyuv.Loop.default_loop()
-        prepare = pyuv.Prepare(loop)
+        prepare = pyuv.Prepare(self.loop)
         prepare.start(prepare_cb)
-        idle = pyuv.Idle(loop)
+        idle = pyuv.Idle(self.loop)
         idle.start(idle_cb)
-        check = pyuv.Check(loop)
+        check = pyuv.Check(self.loop)
         check.start(check_cb)
-        timer = pyuv.Timer(loop)
+        timer = pyuv.Timer(self.loop)
         timer.start(timer_cb, 0.1, 0)
-        loop.run()
+        self.loop.run()
         self.assertEqual(self.prepare_cb_called, 1)
         self.assertEqual(self.idle_cb_called, 1)
         self.assertEqual(self.check_cb_called, 1)
@@ -46,5 +47,4 @@ class MultiHandleTest(unittest2.TestCase):
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
