@@ -3,72 +3,62 @@
 import os
 import socket
 import sys
+import unittest
 
-from common import unittest2
-import common
+from common import linesep, platform_skip, TestCase
 import pyuv
 
-try:
-    memoryview
-except NameError:
-    # Fix for Python 2.6
-    memoryview = str
-
 
 TEST_PORT = 1234
 
-class TCPErrorTest(unittest2.TestCase):
+class TCPErrorTest(TestCase):
 
     def on_client_connect_error(self, client, error):
         self.assertNotEqual(error, None)
         client.close()
 
     def test_client1(self):
-        loop = pyuv.Loop.default_loop()
-        client = pyuv.TCP(loop)
+        client = pyuv.TCP(self.loop)
         client.connect(("127.0.0.1", TEST_PORT), self.on_client_connect_error)
-        loop.run()
+        self.loop.run()
 
     def test_client2(self):
-        loop = pyuv.Loop.default_loop()
-        client = pyuv.TCP(loop)
+        client = pyuv.TCP(self.loop)
         self.assertFalse(client.readable)
         self.assertFalse(client.writable)
         client.close()
-        loop.run()
+        self.loop.run()
 
     def test_open(self):
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        loop = pyuv.Loop.default_loop()
-        client = pyuv.TCP(loop)
+        client = pyuv.TCP(self.loop)
         client.open(sock.fileno())
         client.connect(("127.0.0.1", TEST_PORT), self.on_client_connect_error)
-        loop.run()
+        self.loop.run()
         sock.close()
 
     def test_raise(self):
-        loop = pyuv.Loop.default_loop()
-        tcp = pyuv.TCP(loop)
+        tcp = pyuv.TCP(self.loop)
         self.assertRaises(pyuv.error.TCPError, tcp.write, b"PING")
         tcp.close()
-        loop.run()
+        self.loop.run()
 
 
-class TCPTest(unittest2.TestCase):
+class TCPTest(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTest, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
         self.assertEqual(error, None)
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.write(b"PING"+common.linesep)
+        client.write(b"PING"+linesep)
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -85,7 +75,7 @@ class TCPTest(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        self.assertEqual(data, b"PING"+common.linesep)
+        self.assertEqual(data, b"PING"+linesep)
         client.close()
 
     def test_tcp1(self):
@@ -96,18 +86,24 @@ class TCPTest(unittest2.TestCase):
         self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
         self.loop.run()
 
+    def test_tcp_bind(self):
+        self.server = pyuv.TCP(self.loop)
+        self.server.bind(("", TEST_PORT))
+        self.server.close()
+        self.loop.run()
 
-class TCPTest2(unittest2.TestCase):
+
+class TCPTest2(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTest2, self).setUp()
         self.server = None
         self.client = None
         self.write_cb_count = 0
 
     def on_connection(self, server, error):
         self.assertEqual(error, None)
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         for x in range(1024):
             client.write(b"PING"*1000, self.on_client_write)
@@ -132,16 +128,16 @@ class TCPTest2(unittest2.TestCase):
         self.assertEqual(self.write_cb_count, 1024)
 
 
-class TCPTest3(unittest2.TestCase):
+class TCPTest3(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTest3, self).setUp()
         self.server = None
         self.client = None
 
     def on_connection(self, server, error):
         self.assertEqual(error, None)
-        connection = pyuv.TCP(pyuv.Loop.default_loop())
+        connection = pyuv.TCP(self.loop)
         server.accept(connection)
         while connection.write_queue_size == 0:
             connection.write(b"PING"*1000)
@@ -168,54 +164,17 @@ class TCPTest3(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPTestUnicode(unittest2.TestCase):
+class TCPTestMemoryview(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        self.server = None
-        self.client = None
-
-    def on_connection(self, server, error):
-        self.assertEqual(error, None)
-        client = pyuv.TCP(pyuv.Loop.default_loop())
-        server.accept(client)
-        if sys.version_info >= (3, 0):
-            data = 'PÏNG'
-            exc_type = TypeError
-        else:
-            data = unicode('PÏNG', 'utf-8')
-            exc_type = UnicodeEncodeError
-        self.assertRaises(exc_type, client.write, data)
-        client.close()
-        self.server.close()
-
-    def on_client_connection(self, client, error):
-        self.assertEqual(error, None)
-        client.start_read(self.on_client_read)
-
-    def on_client_read(self, client, data, error):
-        client.close()
-
-    def test_tcp_unicode(self):
-        self.server = pyuv.TCP(self.loop)
-        self.server.bind(("0.0.0.0", TEST_PORT))
-        self.server.listen(self.on_connection)
-        self.client = pyuv.TCP(self.loop)
-        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
-        self.loop.run()
-
-
-class TCPTestMemoryview(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTestMemoryview, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
         self.assertEqual(error, None)
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
@@ -247,21 +206,21 @@ class TCPTestMemoryview(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPTestNull(unittest2.TestCase):
+class TCPTestNull(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTestNull, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
         self.assertEqual(error, None)
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.write(b"PIN\x00G"+common.linesep)
+        client.write(b"PIN\x00G"+linesep)
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -276,7 +235,7 @@ class TCPTestNull(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        self.assertEqual(data, b"PIN\x00G"+common.linesep)
+        self.assertEqual(data, b"PIN\x00G"+linesep)
         client.close()
 
     def test_tcp_null(self):
@@ -288,20 +247,20 @@ class TCPTestNull(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPTestList(unittest2.TestCase):
+class TCPTestList(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTestList, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING6", b"PING7", b"PING8", b"PING9", b"PING10", b"PING11", b"PING12"])
+        client.write([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING6", b"PING7", b"PING8", b"PING9", b"PING10", b"PING11", b"PING12"])
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -328,56 +287,20 @@ class TCPTestList(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPTestListUnicode(unittest2.TestCase):
+class TCPTestListNull(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
-        self.server = None
-        self.client = None
-
-    def on_connection(self, server, error):
-        client = pyuv.TCP(pyuv.Loop.default_loop())
-        server.accept(client)
-        if sys.version_info >= (3, 0):
-            data = 'PÏNG'
-            exc_type = TypeError
-        else:
-            data = unicode('PÏNG', 'utf-8')
-            exc_type = UnicodeEncodeError
-        self.assertRaises(exc_type, client.writelines, [data for x in range(100)])
-        client.close()
-        self.server.close()
-
-    def on_client_connection(self, client, error):
-        self.assertEqual(error, None)
-        client.start_read(self.on_client_read)
-
-    def on_client_read(self, client, data, error):
-        client.close()
-
-    def test_tcp_list_unicode(self):
-        self.server = pyuv.TCP(self.loop)
-        self.server.bind(("0.0.0.0", TEST_PORT))
-        self.server.listen(self.on_connection)
-        self.client = pyuv.TCP(self.loop)
-        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
-        self.loop.run()
-
-
-class TCPTestListNull(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTestListNull, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING\x00\xFF6"])
+        client.write([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING\x00\xFF6"])
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
@@ -404,24 +327,27 @@ class TCPTestListNull(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPTestInvalidData(unittest2.TestCase):
+class TCPTestInvalidData(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPTestInvalidData, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
 
         self.assertRaises(TypeError, client.write, 1234)
         self.assertRaises(TypeError, client.write, object())
-        self.assertRaises(TypeError, client.writelines, 1234)
-        self.assertRaises(TypeError, client.writelines, object())
+        self.assertRaises(TypeError, client.write, u"PING")
+        self.assertRaises(ValueError, client.write, [])
+        self.assertRaises(ValueError, client.write, ())
+        self.assertRaises(TypeError, client.write, list(range(5)))
+        self.assertRaises(TypeError, client.write, [b"PING", 123, u"PING"])
 
         client.close()
         self.client_connections.remove(client)
@@ -451,27 +377,26 @@ class TCPTestInvalidData(unittest2.TestCase):
         self.loop.run()
 
 
-class TCPShutdownTest(unittest2.TestCase):
+class TCPShutdownTest(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(TCPShutdownTest, self).setUp()
         self.server = None
         self.client = None
         self.client_connections = []
 
     def on_connection(self, server, error):
-        client = pyuv.TCP(pyuv.Loop.default_loop())
+        client = pyuv.TCP(self.loop)
         server.accept(client)
         self.client_connections.append(client)
         client.start_read(self.on_client_connection_read)
-        client.write(b"PING"+common.linesep)
+        client.write(b"PING"+linesep)
 
     def on_client_connection_read(self, client, data, error):
         if data is None:
             client.close(self.on_close)
             self.client_connections.remove(client)
             self.server.close(self.on_close)
-            return
 
     def on_client_connection(self, client, error):
         self.assertEqual(error, None)
@@ -486,7 +411,7 @@ class TCPShutdownTest(unittest2.TestCase):
 
     def on_client_read(self, client, data, error):
         self.assertNotEqual(data, None)
-        self.assertEqual(data, b"PING"+common.linesep)
+        self.assertEqual(data, b"PING"+linesep)
         client.shutdown(self.on_client_shutdown)
 
     def test_tcp_shutdown(self):
@@ -502,19 +427,93 @@ class TCPShutdownTest(unittest2.TestCase):
         self.assertEqual(self.close_cb_called, 3)
 
 
-class TCPFlagsTest(unittest2.TestCase):
+class TCPFlagsTest(TestCase):
 
     def test_tcp_flags(self):
-        loop = pyuv.Loop.default_loop()
-        tcp = pyuv.TCP(loop)
+        tcp = pyuv.TCP(self.loop)
         tcp.nodelay(True)
         tcp.keepalive(True, 60)
         tcp.simultaneous_accepts(True)
         tcp.close()
-        loop.run()
-        self.assertTrue(True)
+        self.loop.run()
+
+
+@platform_skip(["win32"])
+class TCPTryTest(TestCase):
+
+    def setUp(self):
+        super(TCPTryTest, self).setUp()
+        self.server = None
+        self.client = None
+        self.connections = None
+
+    def on_connection(self, server, error):
+        self.assertEqual(error, None)
+        client = pyuv.TCP(self.loop)
+        server.accept(client)
+        self.connection = client
+        client.start_read(self.on_client_connection_read)
+        while True:
+            try:
+                r = client.try_write(b"x")
+            except pyuv.error.TCPError:
+                continue
+            if r != 0:
+                break
+
+    def on_client_connection_read(self, client, data, error):
+        if data is None:
+            client.close()
+            self.connection = None
+            self.server.close()
+
+    def on_client_connection(self, client, error):
+        self.assertEqual(error, None)
+        client.start_read(self.on_client_read)
+
+    def on_client_read(self, client, data, error):
+        client.close()
+
+    def test_tcp1_try(self):
+        self.server = pyuv.TCP(self.loop)
+        self.server.bind(("0.0.0.0", TEST_PORT))
+        self.server.listen(self.on_connection)
+        self.client = pyuv.TCP(self.loop)
+        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
+        self.loop.run()
+
+
+class TCPTestFileno(TestCase):
+
+    def check_fileno(self, handle):
+        fd = handle.fileno()
+        self.assertIsInstance(fd, int)
+        self.assertGreaterEqual(fd, 0)
+
+    def on_connection(self, server, error):
+        self.assertEqual(error, None)
+        self.check_fileno(server)
+        client = pyuv.TCP(self.loop)
+        server.accept(client)
+        self.check_fileno(client)
+        client.close()
+        server.close()
+
+    def on_client_connection(self, client, error):
+        self.assertEqual(error, None)
+        self.check_fileno(client)
+        client.close()
+
+    def test_fileno(self):
+        server = pyuv.TCP(self.loop)
+        server.bind(("0.0.0.0", TEST_PORT))
+        self.check_fileno(server)
+        server.listen(self.on_connection)
+        client = pyuv.TCP(self.loop)
+        client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
+        self.check_fileno(client)
+        self.loop.run()
 
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
