@@ -1,19 +1,21 @@
 
-from common import unittest2, platform_skip
-import common
-import pyuv
 import socket
 import sys
+import unittest
+
+from common import linesep, platform_skip, TestCase
+import pyuv
 
 
 TEST_PORT = 12345
 TEST_PORT2 = 12346
 MULTICAST_ADDRESS = "239.255.0.1"
 
-class UDPTest(unittest2.TestCase):
+
+class UDPTest(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPTest, self).setUp()
         self.server = None
         self.client = None
 
@@ -25,7 +27,7 @@ class UDPTest(unittest2.TestCase):
         ip, port = ip_port
         data = data.strip()
         self.assertEqual(data, b"PING")
-        self.server.send((ip, port), b"PONG"+common.linesep)
+        self.server.send((ip, port), b"PONG"+linesep)
 
     def on_client_recv(self, handle, ip_port, flags, data, error):
         self.assertEqual(flags, 0)
@@ -36,7 +38,7 @@ class UDPTest(unittest2.TestCase):
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send(("127.0.0.1", TEST_PORT), b"PING"+common.linesep)
+        self.client.send(("127.0.0.1", TEST_PORT), b"PING"+linesep)
         timer.close(self.on_close)
 
     def test_udp_pingpong(self):
@@ -59,10 +61,39 @@ class UDPTest(unittest2.TestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestNull(unittest2.TestCase):
+class UDPEmptyDatagramTest(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPEmptyDatagramTest, self).setUp()
+        self.server = None
+        self.client = None
+        self.on_close_called = 0
+
+    def on_close(self, handle):
+        self.on_close_called += 1
+
+    def on_client_recv(self, handle, ip_port, flags, data, error):
+        self.assertEqual(flags, 0)
+        ip, port = ip_port
+        self.assertEqual(error, None)
+        self.assertEqual(data, b"")
+        self.client.close(self.on_close)
+        self.server.close(self.on_close)
+
+    def test_udp_empty_datagram(self):
+        self.server = pyuv.UDP(self.loop)
+        self.client = pyuv.UDP(self.loop)
+        self.client.bind(("0.0.0.0", TEST_PORT2))
+        self.client.start_recv(self.on_client_recv)
+        self.server.send(("127.0.0.1", TEST_PORT2), b"")
+        self.loop.run()
+        self.assertEqual(self.on_close_called, 2)
+
+
+class UDPTestNull(TestCase):
+
+    def setUp(self):
+        super(UDPTestNull, self).setUp()
         self.server = None
         self.client = None
 
@@ -74,7 +105,7 @@ class UDPTestNull(unittest2.TestCase):
         ip, port = ip_port
         data = data.strip()
         self.assertEqual(data, b"PIN\x00G")
-        self.server.send((ip, port), b"PONG"+common.linesep)
+        self.server.send((ip, port), b"PONG"+linesep)
 
     def on_client_recv(self, handle, ip_port, flags, data, error):
         self.assertEqual(flags, 0)
@@ -85,7 +116,7 @@ class UDPTestNull(unittest2.TestCase):
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.send(("127.0.0.1", TEST_PORT), b"PIN\x00G"+common.linesep)
+        self.client.send(("127.0.0.1", TEST_PORT), b"PIN\x00G"+linesep)
         timer.close(self.on_close)
 
     def test_udp_pingpong_null(self):
@@ -102,10 +133,10 @@ class UDPTestNull(unittest2.TestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestList(unittest2.TestCase):
+class UDPTestList(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPTestList, self).setUp()
         self.server = None
         self.client = None
 
@@ -117,7 +148,7 @@ class UDPTestList(unittest2.TestCase):
         ip, port = ip_port
         data = data.strip()
         self.assertEqual(data, b"PING")
-        self.server.sendlines((ip, port), [b"PONG", common.linesep])
+        self.server.send((ip, port), [b"PONG", linesep])
 
     def on_client_recv(self, handle, ip_port, flags, data, error):
         self.assertEqual(flags, 0)
@@ -128,7 +159,7 @@ class UDPTestList(unittest2.TestCase):
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.sendlines(("127.0.0.1", TEST_PORT), [b"PING", common.linesep])
+        self.client.send(("127.0.0.1", TEST_PORT), [b"PING", linesep])
         timer.close(self.on_close)
 
     def test_udp_pingpong_list(self):
@@ -145,10 +176,10 @@ class UDPTestList(unittest2.TestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestListNull(unittest2.TestCase):
+class UDPTestListNull(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPTestListNull, self).setUp()
         self.server = None
         self.client = None
 
@@ -160,7 +191,7 @@ class UDPTestListNull(unittest2.TestCase):
         ip, port = ip_port
         data = data.strip()
         self.assertEqual(data, b"PIN\x00G")
-        self.server.sendlines((ip, port), [b"PONG", common.linesep])
+        self.server.send((ip, port), [b"PONG", linesep])
 
     def on_client_recv(self, handle, ip_port, flags, data, error):
         self.assertEqual(flags, 0)
@@ -171,7 +202,7 @@ class UDPTestListNull(unittest2.TestCase):
         self.server.close(self.on_close)
 
     def timer_cb(self, timer):
-        self.client.sendlines(("127.0.0.1", TEST_PORT), [b"PIN\x00G", common.linesep])
+        self.client.send(("127.0.0.1", TEST_PORT), [b"PIN\x00G", linesep])
         timer.close(self.on_close)
 
     def test_udp_pingpong_list_null(self):
@@ -188,10 +219,10 @@ class UDPTestListNull(unittest2.TestCase):
         self.assertEqual(self.on_close_called, 3)
 
 
-class UDPTestInvalidData(unittest2.TestCase):
+class UDPTestInvalidData(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPTestInvalidData, self).setUp()
         self.server = None
         self.client = None
 
@@ -208,8 +239,8 @@ class UDPTestInvalidData(unittest2.TestCase):
     def timer_cb(self, timer):
         self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), object())
         self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), 1)
-        self.assertRaises(TypeError, self.client.sendlines, ("127.0.0.1", TEST_PORT), object())
-        self.assertRaises(TypeError, self.client.sendlines, ("127.0.0.1", TEST_PORT), 1)
+        self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), object())
+        self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), 1)
 
         self.client.close(self.on_close)
         self.server.close(self.on_close)
@@ -238,10 +269,10 @@ else:
         yield MULTICAST_ADDRESS
 
 
-class UDPTestMulticast(unittest2.TestCase):
+class UDPTestMulticast(TestCase):
 
     def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+        super(UDPTestMulticast, self).setUp()
         self.server = None
         self.clients = None
         self.received_data = None
@@ -294,10 +325,7 @@ class UDPTestMulticast(unittest2.TestCase):
         self.assertEqual(self.received_data, b"PING")
 
 
-class UDPTestBigDatagram(unittest2.TestCase):
-
-    def setUp(self):
-        self.loop = pyuv.Loop.default_loop()
+class UDPTestBigDatagram(TestCase):
 
     def send_cb(self, handle, error):
         self.handle.close()
@@ -312,21 +340,113 @@ class UDPTestBigDatagram(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_EMSGSIZE)
 
 
-class UDPTestOpen(unittest2.TestCase):
+class UDPTestOpen(TestCase):
 
     def test_udp_open(self):
         sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        loop = pyuv.Loop.default_loop()
-        handle = pyuv.UDP(loop)
+        handle = pyuv.UDP(self.loop)
         handle.open(sock.fileno())
         try:
             handle.bind(("1.2.3.4", TEST_PORT))
         except pyuv.error.UDPError as e:
             self.assertEqual(e.args[0], pyuv.errno.UV_EADDRNOTAVAIL)
-        loop.run()
+        self.loop.run()
         sock.close()
 
 
-if __name__ == '__main__':
-    unittest2.main(verbosity=2)
+class UDPTestBind(TestCase):
 
+    def test_udp_bind(self):
+        handle = pyuv.UDP(self.loop)
+        handle.bind(("", TEST_PORT))
+        handle.close()
+        self.loop.run()
+
+    def test_udp_bind_reuse(self):
+        handle = pyuv.UDP(self.loop)
+        handle.bind(("", TEST_PORT), pyuv.UV_UDP_REUSEADDR)
+        handle2 = pyuv.UDP(self.loop)
+        handle2.bind(("", TEST_PORT), pyuv.UV_UDP_REUSEADDR)
+        handle.close()
+        handle2.close()
+        self.loop.run()
+
+    def test_udp_bind_noreuse(self):
+        handle = pyuv.UDP(self.loop)
+        handle.bind(("", TEST_PORT))
+        handle2 = pyuv.UDP(self.loop)
+        self.assertRaises(pyuv.error.UDPError, handle2.bind, ("", TEST_PORT))
+        handle.close()
+        handle2.close()
+        self.loop.run()
+
+
+class UDPTestFileno(TestCase):
+
+    def check_fileno(self, handle):
+        fd = handle.fileno()
+        self.assertIsInstance(fd, int)
+        self.assertGreaterEqual(fd, 0)
+
+    def test_udp_fileno(self):
+        server = pyuv.UDP(self.loop)
+        server.bind(("0.0.0.0", TEST_PORT))
+        self.check_fileno(server)
+        server.close()
+
+
+class UDPTestMulticastInterface(TestCase):
+
+    def test_udp_multicast_interface(self):
+        handle = pyuv.UDP(self.loop)
+        handle.bind(("", TEST_PORT))
+        handle.set_multicast_interface("0.0.0.0")
+        handle.close()
+        self.loop.run()
+
+
+@platform_skip(["win32"])
+class UDPTryTest(TestCase):
+
+    def setUp(self):
+        super(UDPTryTest, self).setUp()
+        self.server = None
+        self.client = None
+
+    def on_close(self, handle):
+        self.on_close_called += 1
+
+    def on_server_recv(self, handle, ip_port, flags, data, error):
+        self.assertEqual(flags, 0)
+        ip, port = ip_port
+        data = data.strip()
+        self.assertEqual(data, b"PING")
+        self.server.close(self.on_close)
+        self.client.close(self.on_close)
+
+    def timer_cb(self, timer):
+        timer.close(self.on_close)
+        while True:
+            try:
+                r = self.client.try_send(("127.0.0.1", TEST_PORT), b"PING")
+            except pyuv.error.UDPError as e:
+                self.assertEqual(e.args[0], pyuv.errno.UV_EAGAIN)
+            else:
+                self.assertEqual(r, 4)
+                break
+
+    def test_udp_try_send(self):
+        self.on_close_called = 0
+        self.server = pyuv.UDP(self.loop)
+        self.server.bind(("0.0.0.0", TEST_PORT))
+        self.server.start_recv(self.on_server_recv)
+        self.client = pyuv.UDP(self.loop)
+        self.client.bind(("0.0.0.0", TEST_PORT2))
+        timer = pyuv.Timer(self.loop)
+        timer.start(self.timer_cb, 0.1, 0)
+        self.loop.run()
+        self.assertEqual(self.on_close_called, 3)
+
+
+if __name__ == '__main__':
+    unittest.main(verbosity=2)
