@@ -1,11 +1,11 @@
 
-from common import unittest2
+import unittest
 
+from common import TestCase
 import pyuv
-import socket
 
 
-class UtilTest(unittest2.TestCase):
+class UtilTest(TestCase):
 
     def test_hrtime(self):
         r = pyuv.util.hrtime()
@@ -39,14 +39,6 @@ class UtilTest(unittest2.TestCase):
         r = pyuv.util.cpu_info()
         self.assertTrue(r)
 
-    def test_getaddrinfo(self):
-        def getaddrinfo_cb(result, errorno):
-            self.assertEqual(errorno, None)
-        loop = pyuv.Loop.default_loop()
-        pyuv.util.getaddrinfo(loop, getaddrinfo_cb, 'localhost', 80, socket.AF_INET)
-        loop.run()
-
 
 if __name__ == '__main__':
-    unittest2.main(verbosity=2)
-
+    unittest.main(verbosity=2)
